@@ -1,7 +1,8 @@
-from typing import Any, Callable, List, Optional, Type, TypeVar
+from typing import Any, Callable, List, Literal, Optional, TypeVar
 
 import twitter_openapi_python_generated as twitter
 import twitter_openapi_python_generated.models as models
+from x_client_transaction import ClientTransaction
 
 from twitter_openapi_python.models import (
     ApiUtilsRaw,
@@ -11,8 +12,8 @@ from twitter_openapi_python.models import (
 )
 from twitter_openapi_python.utils import (
     build_response,
-    check_error,
     entries_cursor,
+    error_check,
     get_kwargs,
     instruction_to_entry,
     tweet_entries_converter,
@@ -20,7 +21,7 @@ from twitter_openapi_python.utils import (
 
 T = TypeVar("T")
 ResponseType = TwitterApiUtilsResponse[TimelineApiUtilsResponse[TweetApiUtilsData]]
-ApiFnType = Callable[..., twitter.ApiResponse]
+ApiFnType = Callable[..., twitter.ApiResponse[T]]
 ParamType = dict[str, Any]
 
 
@@ -28,23 +29,21 @@ class TweetApiUtils:
     api: twitter.TweetApi
     flag: ParamType
 
-    def __init__(self, api: twitter.TweetApi, flag: ParamType):
+    def __init__(self, api: twitter.TweetApi, flag: ParamType, ct: ClientTransaction):
         self.api = api
         self.flag = flag
+        self.ct = ct
 
     def request(
         self,
-        apiFn: ApiFnType,
+        apiFn: "ApiFnType[T]",
         convertFn: Callable[[T], List[models.InstructionUnion]],
-        type: Type[T],
         key: str,
         param: ParamType,
     ) -> ResponseType:
-        args = get_kwargs(flag=self.flag[key], additional=param)
+        args = get_kwargs(flag=self.flag[key], additional=param, ct=self.ct)
         res = apiFn(**args)
-        checked = check_error(data=res, type=type)
-
-        instruction = convertFn(checked)
+        instruction = convertFn(res.data)
         entry = instruction_to_entry(instruction)
         tweet_list = tweet_entries_converter(entry)
         raw = ApiUtilsRaw(
@@ -56,8 +55,7 @@ class TweetApiUtils:
             cursor=entries_cursor(entry),
             data=tweet_list,
         )
-
-        return build_response(response=res, data=data)
+        return build_response(res, data)
 
     def get_tweet_detail(
         self,
@@ -76,8 +74,7 @@ class TweetApiUtils:
 
         response = self.request(
             apiFn=self.api.get_tweet_detail_with_http_info,
-            convertFn=lambda e: e.data.threaded_conversation_with_injections_v2.instructions,
-            type=models.TweetDetailResponse,
+            convertFn=lambda e: error_check(e.data.threaded_conversation_with_injections_v2, e.errors).instructions,
             key="TweetDetail",
             param=param,
         )
@@ -86,7 +83,7 @@ class TweetApiUtils:
     def get_search_timeline(
         self,
         raw_query: str,
-        product: Optional[str] = None,
+        product: Optional[Literal["Top", "Latest", "People", "Photos", "Videos"]] = None,
         cursor: Optional[str] = None,
         count: Optional[int] = None,
         extra_param: Optional[ParamType] = None,
@@ -103,8 +100,7 @@ class TweetApiUtils:
 
         response = self.request(
             apiFn=self.api.get_search_timeline_with_http_info,
-            convertFn=lambda e: e.data.search_by_raw_query.search_timeline.timeline.instructions,
-            type=models.SearchTimelineResponse,
+            convertFn=lambda e: error_check(e.data.search_by_raw_query, e.errors).search_timeline.timeline.instructions,
             key="SearchTimeline",
             param=param,
         )
@@ -126,8 +122,7 @@ class TweetApiUtils:
 
         response = self.request(
             apiFn=self.api.get_home_timeline_with_http_info,
-            convertFn=lambda e: e.data.home.home_timeline_urt.instructions,
-            type=models.TimelineResponse,
+            convertFn=lambda e: error_check(e.data.home, e.errors).home_timeline_urt.instructions,
             key="HomeTimeline",
             param=param,
         )
@@ -149,8 +144,7 @@ class TweetApiUtils:
 
         response = self.request(
             apiFn=self.api.get_home_latest_timeline_with_http_info,
-            convertFn=lambda e: e.data.home.home_timeline_urt.instructions,
-            type=models.TimelineResponse,
+            convertFn=lambda e: error_check(e.data.home, e.errors).home_timeline_urt.instructions,
             key="HomeLatestTimeline",
             param=param,
         )
@@ -173,8 +167,9 @@ class TweetApiUtils:
 
         response = self.request(
             apiFn=self.api.get_list_latest_tweets_timeline_with_http_info,
-            convertFn=lambda e: e.data.list.tweets_timeline.timeline.instructions,
-            type=models.ListLatestTweetsTimelineResponse,
+            convertFn=lambda e: error_check(
+                error_check(e.data.list, e.errors).tweets_timeline.timeline, e.errors
+            ).instructions,
             key="ListLatestTweetsTimeline",
             param=param,
         )
@@ -197,8 +192,9 @@ class TweetApiUtils:
 
         response = self.request(
             apiFn=self.api.get_user_tweets_with_http_info,
-            convertFn=lambda e: e.data.user.result.timeline_v2.timeline.instructions,
-            type=models.UserTweetsResponse,
+            convertFn=lambda e: error_check(
+                error_check(e.data.user, e.errors).result.timeline.timeline, e.errors
+            ).instructions,
             key="UserTweets",
             param=param,
         )
@@ -221,8 +217,9 @@ class TweetApiUtils:
 
         response = self.request(
             apiFn=self.api.get_user_tweets_and_replies_with_http_info,
-            convertFn=lambda e: e.data.user.result.timeline_v2.timeline.instructions,
-            type=models.UserTweetsResponse,
+            convertFn=lambda e: error_check(
+                error_check(e.data.user, e.errors).result.timeline.timeline, e.errors
+            ).instructions,
             key="UserTweetsAndReplies",
             param=param,
         )
@@ -245,8 +242,9 @@ class TweetApiUtils:
 
         response = self.request(
             apiFn=self.api.get_user_media_with_http_info,
-            convertFn=lambda e: e.data.user.result.timeline_v2.timeline.instructions,
-            type=models.UserTweetsResponse,
+            convertFn=lambda e: error_check(
+                error_check(e.data.user, e.errors).result.timeline.timeline, e.errors
+            ).instructions,
             key="UserMedia",
             param=param,
         )
@@ -269,8 +267,9 @@ class TweetApiUtils:
 
         response = self.request(
             apiFn=self.api.get_likes_with_http_info,
-            convertFn=lambda e: e.data.user.result.timeline_v2.timeline.instructions,
-            type=models.UserTweetsResponse,
+            convertFn=lambda e: error_check(
+                error_check(e.data.user, e.errors).result.timeline.timeline, e.errors
+            ).instructions,
             key="Likes",
             param=param,
         )
@@ -292,9 +291,61 @@ class TweetApiUtils:
 
         response = self.request(
             apiFn=self.api.get_bookmarks_with_http_info,
-            convertFn=lambda e: e.data.bookmark_timeline_v2.timeline.instructions,
-            type=models.BookmarksResponse,
+            convertFn=lambda e: error_check(
+                error_check(e.data, e.errors).bookmark_timeline_v2.timeline, e.errors
+            ).instructions,
             key="Bookmarks",
             param=param,
         )
         return response
+
+    def get_community_tweets_timeline(
+        self,
+        cursor: Optional[str] = None,
+        count: Optional[int] = None,
+        rankingMode: Optional[Literal["Recency", "Relevance"]] = None,
+        extra_param: Optional[ParamType] = None,
+    ) -> ResponseType:
+        param: ParamType = {}
+        if cursor is not None:
+            param["cursor"] = cursor
+        if count is not None:
+            param["count"] = count
+        if rankingMode is not None:
+            param["rankingMode"] = rankingMode
+        if extra_param is not None:
+            param.update(extra_param)
+
+        response = self.request(
+            apiFn=self.api.get_community_tweets_timeline_with_http_info,
+            convertFn=lambda e: error_check(
+                error_check(e.data, e.errors).community_results.result.ranked_community_timeline.timeline, e.errors
+            ).instructions,
+            key="CommunityTweetsTimeline",
+            param=param,
+        )
+        return response
+
+    def get_community_media_timeline(
+        self,
+        cursor: Optional[str] = None,
+        count: Optional[int] = None,
+        extra_param: Optional[ParamType] = None,
+    ) -> ResponseType:
+        param: ParamType = {}
+        if cursor is not None:
+            param["cursor"] = cursor
+        if count is not None:
+            param["count"] = count
+        if extra_param is not None:
+            param.update(extra_param)
+
+        response = self.request(
+            apiFn=self.api.get_community_media_timeline_with_http_info,
+            convertFn=lambda e: error_check(
+                error_check(e.data, e.errors).community_results.result.community_media_timeline.timeline, e.errors
+            ).instructions,
+            key="CommunityMediaTimeline",
+            param=param,
+        )
+        return response
